@@ -1,52 +1,39 @@
 package Biber::Constants;
-use v5.24;
+use v5.16;
 use strict;
 use warnings;
 
-use Encode;
 use Encode::Alias;
 
-use parent qw(Exporter);
-use Biber::Date::Format;
-use Text::CSV;
+use base 'Exporter';
 
 our @EXPORT = qw{
                   $CONFIG_DEFAULT_BIBER
-                  $CONFIG_CSV_PARSER
-                  $BIBER_CONF_NAME
-                  $BCF_VERSION
-                  $BBL_VERSION
-                  $BIBER_SORT_FINAL
-                  $BIBER_SUPPRESS_FINAL
-                  $BIBER_SORT_NULL
-                  $LABEL_FINAL
                   %CONFIG_DEFAULT_BIBLATEX
-                  %CONFIG_OPTSCOPE_BIBLATEX
-                  %CONFIG_SCOPEOPT_BIBLATEX
-                  %CONFIG_OPTTYPE_BIBLATEX
-                  %CONFIG_BIBLATEX_OPTIONS
-                  %CONFIG_META_MARKERS
-                  %CONFIG_DATE_PARSERS
-                  %DATAFIELD_SETS
+                  %CONFIG_SCOPE_BIBLATEX
+                  %CONFIG_BIBLATEX_PER_ENTRY_OPTIONS
+                  %NOSORT_TYPES
                   %DM_DATATYPES
                   %LOCALE_MAP
                   %LOCALE_MAP_R
                   %REMOTE_MAP
-                  %DS_EXTENSIONS
-                  %UNIQUENAME_CONTEXTS
-                  %UNIQUENAME_VALUES
-                  %MONTHS
+                  $BIBER_CONF_NAME
+                  $BCF_VERSION
+                  $BBL_VERSION
+                  $BIBER_SORT_FINAL
+                  $BIBER_SORT_NULL
+                  $LABEL_FINAL
               };
 
 # Version of biblatex control file which this release expects. Matched against version
 # passed in control file. Used when checking the .bcf
-our $BCF_VERSION = '3.7';
+our $BCF_VERSION = '2.9';
 # Format version of the .bbl. Used when writing the .bbl
-our $BBL_VERSION = '3.1';
+our $BBL_VERSION = '2.5';
 
 # Global flags needed for sorting
-our $BIBER_SORT_FINAL;
-our $BIBER_SORT_NULL;
+our $BIBER_SORT_FINAL = 0;
+our $BIBER_SORT_NULL  = 0;
 
 # the name of the Biber configuration file, which should be
 # either returned by kpsewhich or located at "$HOME/.$BIBER_CONF_NAME"
@@ -65,143 +52,105 @@ unless ($locale) {
   }
 }
 
-our %MONTHS = ('jan' => '1',
-              'feb' => '2',
-              'mar' => '3',
-              'apr' => '4',
-              'may' => '5',
-              'jun' => '6',
-              'jul' => '7',
-              'aug' => '8',
-              'sep' => '9',
-              'oct' => '10',
-              'nov' => '11',
-              'dec' => '12');
-
-# datafieldsets
-our %DATAFIELD_SETS = ();
+# nosort type category shortcuts
+our %NOSORT_TYPES = (
+                     type_name => {
+                                   author => 1,
+                                   afterword => 1,
+                                   annotator => 1,
+                                   bookauthor => 1,
+                                   commentator => 1,
+                                   editor => 1,
+                                   editora => 1,
+                                   editorb => 1,
+                                   editorc => 1,
+                                   foreword => 1,
+                                   holder => 1,
+                                   introduction => 1,
+                                   namea => 1,
+                                   nameb => 1,
+                                   namec => 1,
+                                   shortauthor => 1,
+                                   shorteditor => 1,
+                                   translator => 1
+                                  },
+                     type_title => {
+                                    booktitle => 1,
+                                    eventtitle => 1,
+                                    issuetitle => 1,
+                                    journaltitle => 1,
+                                    maintitle => 1,
+                                    origtitle => 1,
+                                    title => 1
+                                   }
+);
 
 # datatypes for data model validation
 our %DM_DATATYPES = (
-                     integer  => qr/\A\d+\z/xms,
+                     integer => qr/\A\d+\z/xms,
                      datepart => qr/\A\d+\z/xms
-                    );
-
-# Mapping of data source and output types to extensions
-our %DS_EXTENSIONS = (
-                      bbl        => 'bbl',
-                      bblxml     => 'bblxml',
-                      bibtex     => 'bib',
-                      biblatexml => 'bltxml'
-                      );
-
-# Mapping of biblatex uniquename option to disambiguation level
-our %UNIQUENAME_CONTEXTS = ('false' => 'none',
-                            'init' => 'init',
-                            'full' => 'initorfull',
-                            'allinit' => 'init',
-                            'allfull' => 'initorfull',
-                            'mininit' => 'init',
-                            'minfull' => 'initorfull');
-
-# Mapping of strings to numeric uniquename values for easier biblatex processing
-our %UNIQUENAME_VALUES = ('none' => 0, 'init' => 1, full => 2);
+);
 
 # Biber option defaults. Mostly not needed outside of tool mode since they are passed by .bcf
+
 our $CONFIG_DEFAULT_BIBER = {
-  annotation_marker                           => { content => q/+an/ },
-  clrmacros                                   => { content => 0 },
-  collate_options                             => { option => {level => 4, variable => 'non-ignorable', normalization => 'prenormalized' }},
-  graph                                       => { content => 0 },
-  debug                                       => { content => 0 },
-  dieondatamodel                              => { content => 0 },
-  decodecharsset                              => { content => 'base' },
-  dot_include                                 => { option => {section => 1, xdata => 1, crossref => 1, xref => 1 }},
-  fixinits                                    => { content => 0 },
-  input_encoding                              => { content => 'UTF-8' },
-  input_format                                => { content => 'bibtex' },
-  isbn10                                      => { content => 0 },
-  isbn13                                      => { content => 0 },
-  isbn_normalise                              => { content => 0 },
-  listsep                                     => { content => 'and' },
-  mincrossrefs                                => { content => 2 },
-  minxrefs                                    => { content => 2 },
-  named_annotation_marker                     => { content => q/:/ },
-  namesep                                     => { content => 'and' },
-  no_bblxml_schema                            => { content => 0 },
-  no_bltxml_schema                            => { content => 0 },
-  nodieonerror                                => { content => 0 },
-  noinit                                      => { option => [ {value => q/\b\p{Ll}{2}\p{Pd}(?=\S)/},
-                                                               {value => q/[\x{2bf}\x{2018}]/} ] },
-  nolabel                                     => { option => [ {value => q/[\p{Pc}\p{Ps}\p{Pe}\p{Pi}\p{Pf}\p{Po}\p{S}\p{C}]+/} ] },
-#  nolabelwidthcount                          => { option =>  }, # default is nothing
-  nolog                                       => { content => 0 },
-  noskipduplicates                            => { content => 0 },
-  nostdmacros                                 => { content => 0 },
-  nosort                                      => { option => [ { name => 'setnames', value => q/\A\p{L}{2}\p{Pd}(?=\S)/ },
-                                                               { name => 'setnames', value => q/[\x{2bf}\x{2018}]/ } ] },
-  onlylog                                     => { content => 0 },
-  others_string                               => { content => 'others' },
-  output_align                                => { content => 0 },
-  output_annotation_marker                    => { content => '+an' },
-  output_named_annotation_marker              => { content => ':' },
-  output_encoding                             => { content => 'UTF-8' },
-  output_field_order                          => { content => 'options,abstract,names,lists,dates' },
-  output_format                               => { content => 'bbl' },
-  output_indent                               => { content => '2' },
-  output_fieldcase                            => { content => 'upper' },
-  output_listsep                              => { content => 'and' },
-  output_namesep                              => { content => 'and' },
-  output_resolve_xdata                        => { content => 0 },
-  output_resolve_crossrefs                    => { content => 0 },
-  output_resolve_sets                         => { content => 0 },
-  output_safechars                            => { content => 0 },
-  output_safecharsset                         => { content => 'base' },
-  output_xdatamarker                          => { content => 'xdata' },
-  output_xdatasep                             => { content => '-' },
-  output_xnamesep                             => { content => '=' },
-  quiet                                       => { content => 0 },
-  remove_tmp_dir                              => { content => 1 },
-  sortdebug                                   => { content => 0 },
-  sortcase                                    => { content => 1 },
-  sortupper                                   => { content => 1 },
-  strip_comments                              => { content => 0 },
-  tool                                        => { content => 0 },
-  tool_noremove_missing_dependants            => { content => 0 },
-  trace                                       => { content => 0 },
-  nouri_encode                                => { content => 0 },
-  validate_bblxml                             => { content => 0 },
-  validate_bltxml                             => { content => 0 },
-  validate_config                             => { content => 0 },
-  validate_control                            => { content => 0 },
-  validate_datamodel                          => { content => 0 },
-  wraplines                                   => { content => 0 },
-  xdatamarker                                 => { content => 'xdata' },
-  xdatasep                                    => { content => '-' },
-  xnamesep                                    => { content => '=' },
-  xsvsep                                      => { content => q/\s*,\s*/ },
+  clrmacros           => { content => 0 },
+  collate             => { content => 1 },
+  collate_options     => { option => {level => 4, variable => 'non-ignorable', normalization => 'prenormalized' }},
+  graph               => { content => 0 },
+  debug               => { content => 0 },
+  decodecharsset      => { content => 'base' },
+  dot_include         => { option => {section => 1, xdata => 1, crossref => 1, xref => 1 }},
+  fastsort            => { content => 0 },
+  fixinits            => { content => 0 },
+  input_encoding      => { content => 'UTF-8' },
+  input_format        => { content => 'bibtex' },
+  isbn10              => { content => 0 },
+  isbn13              => { content => 0 },
+  isbn_normalise      => { content => 0 },
+  listsep             => { content => 'and' },
+  mincrossrefs        => { content => 2 },
+  namesep             => { content => 'and' },
+  nodieonerror        => { content => 0 },
+  noinit              => { option => [ {value => q/\b\p{Ll}{2}\p{Pd}/},
+                                       {value => q/[\x{2bf}\x{2018}]/} ] },
+  nolabel             => { option => [ {value => q/[\p{P}\p{S}\p{C}]+/} ] },
+  nolabelwidthcount   => { option => [ {value => q//} ] },
+  nolog               => { content => 0 },
+  nostdmacros         => { content => 0 },
+  nosort              => { option => [ { name => 'type_name', value => q/\A\p{L}{2}\p{Pd}/ },
+                                       { name => 'type_name', value => q/[\x{2bf}\x{2018}]/ } ] },
+  onlylog             => { content => 0 },
+  others_string       => { content => 'others' },
+  output_align        => { content => 0 },
+  output_encoding     => { content => 'UTF-8' },
+  output_format       => { content => 'bbl' },
+  output_indent       => { content => '2' },
+  output_fieldcase    => { content => 'upper' },
+  output_resolve      => { content => 0 },
+  output_safechars    => { content => 0 },
+  output_safecharsset => { content => 'base' },
+  quiet               => { content => 0 },
+  noskipduplicates    => { content => 0 },
+  sortcase            => { content => 1 },
+  sortfirstinits      => { content => 0 },
+  sortupper           => { content => 1 },
+  strip_comments      => { content => 0 },
+  tool                => { content => 0 },
+  trace               => { content => 0 },
+  validate_config     => { content => 0 },
+  validate_control    => { content => 0 },
+  validate_datamodel  => { content => 0 },
+  wraplines           => { content => 0 },
+  xsvsep              => { content => q/\s*,\s*/ },
 };
 
-# Set up some re-usable CSV parsers here for efficiency reasons
-our $CONFIG_CSV_PARSER = Text::CSV->new ( { binary           => 1,
-                                            allow_whitespace => 1,
-                                            always_quote     => 1  } );
-
-# Set up some re-usable Date parsers here for efficiency reasons
-# We need two as the missing component data is in these objects, not
-# in the DT objects returned by ->parse_datetime() and this data will
-# likely be different for start/end
-our %CONFIG_DATE_PARSERS = ('start' => Biber::Date::Format->new(),
-                            'end'   => Biber::Date::Format->new());
-
-our %CONFIG_META_MARKERS = ();
-
 # default global options for biblatex
 # Used to set:
 # * Some tool-mode defaults (as there is no .bcf and some biblatex options
 #   cannot be set in a biber config file)
 our %CONFIG_DEFAULT_BIBLATEX = (
-                                sortingtemplatename    => 'tool',
+                                sortscheme    => 'none',
                                 useauthor     => 1,
                                 useeditor     => 1,
                                 usetranslator => 1,
@@ -210,10 +159,8 @@ our %CONFIG_DEFAULT_BIBLATEX = (
                                 minbibnames   => 100,
                                 maxalphanames => 100,
                                 maxcitenames  => 100,
-                                maxsortnames  => 100,
                                 minalphanames => 100,
                                 mincitenames  => 100,
-                                minsortnames  => 100,
                                 minitems      => 100,
                                 useprefix     => 0
                                );
@@ -291,7 +238,7 @@ our %LOCALE_MAP = (
                    'japanese'        => 'ja-JP',
                    'kannada'         => 'kn-IN',
                    'lao'             => 'lo-LA',
-                   'latin'           => 'la-Latn',
+                   'latin'           => 'sr-Latn',
                    'latvian'         => 'lv-LV',
                    'lithuanian'      => 'lt-LT',
                    'lowersorbian'    => 'dsb-DE',
@@ -322,7 +269,7 @@ our %LOCALE_MAP = (
                    'samin'           => 'se-NO',
                    'sanskrit'        => 'sa-IN',
                    'scottish'        => 'gd-GB',
-                   'serbian'         => 'sr-Latn',
+                   'serbian'         => 'sr-Cyrl',
                    'serbianc'        => 'sr-Cyrl',
                    'slovak'          => 'sk-SK',
                    'slovene'         => 'sl-SI',
@@ -517,15 +464,40 @@ our %LOCALE_MAP_R = (
                      'vi-VN'      => 'vietnamese',
                     );
 
-# Holds the scope of each of the BibLaTeX configuration options from the .bcf
-our %CONFIG_OPTSCOPE_BIBLATEX;
-# Holds the options in a particular scope
-our %CONFIG_SCOPEOPT_BIBLATEX;
-# Holds the datatype of an option at a particular scope
-our %CONFIG_OPTTYPE_BIBLATEX;
-# For per-entry, per-namelist and per-name options, what should be set when we find them and
-# should they be output to the .bbl for biblatex.
-our %CONFIG_BIBLATEX_OPTIONS;
+# Holds the scope of each of the BibLaTeX configuration options fro the .bcf
+our %CONFIG_SCOPE_BIBLATEX;
+
+# For per-entry options, what should be set when we find them and
+# what should be output to the .bbl for biblatex.
+# Basically, here we have to emulate relevant parts of biblatex's options processing
+# for local entry-specific options, note therefore the presence here of some
+# options like max/mincitenames which are not passed in the .bcf
+our %CONFIG_BIBLATEX_PER_ENTRY_OPTIONS =  (
+  dataonly       => {OUTPUT => 1, INPUT => {'skiplab' => 1,
+                                            'skipbiblist' => 1,
+                                            'uniquename' => 0,
+                                            'uniquelist' => 0}},
+  maxitems       => {OUTPUT => 1},
+  minitems       => {OUTPUT => 1},
+  maxbibnames    => {OUTPUT => 1},
+  minbibnames    => {OUTPUT => 1},
+  maxcitenames   => {OUTPUT => 1},
+  mincitenames   => {OUTPUT => 1},
+  maxalphanames  => {OUTPUT => 0},
+  minalphanames  => {OUTPUT => 0},
+  maxnames       => {OUTPUT => ['maxcitenames', 'maxbibnames'], INPUT  => ['maxcitenames', 'maxbibnames']},
+  minnames       => {OUTPUT => ['mincitenames', 'minbibnames'], INPUT  => ['mincitenames', 'minbibnames']},
+  presort        => {OUTPUT => 0},
+  skipbib        => {OUTPUT => 1},
+  skipbiblist    => {OUTPUT => 1},
+  skiplab        => {OUTPUT => 1},
+  uniquelist     => {OUTPUT => 0},
+  useauthor      => {OUTPUT => 1},
+  useeditor      => {OUTPUT => 1},
+  useprefix      => {OUTPUT => 1},
+  usetranslator  => {OUTPUT => 1},
+);
+
 
 1;
 
@@ -541,6 +513,7 @@ Biber::Constants - global constants for biber
 
 =head1 AUTHOR
 
+François Charette, C<< <firmicus at ankabut.net> >>
 Philip Kime C<< <philip at kime.org.uk> >>
 
 =head1 BUGS
@@ -550,8 +523,7 @@ L<https://github.com/plk/biber/issues>.
 
 =head1 COPYRIGHT & LICENSE
 
-Copyright 2009-2012 François Charette and Philip Kime, all rights reserved.
-Copyright 2012-2019 Philip Kime, all rights reserved.
+Copyright 2009-2015 François Charette and Philip Kime, all rights reserved.
 
 This module is free software.  You can redistribute it and/or
 modify it under the terms of the Artistic License 2.0.
