@@ -1,5 +1,5 @@
 package Biber::Entry;
-use v5.24;
+use v5.16;
 use strict;
 use warnings;
 
@@ -8,9 +8,9 @@ use Biber::Internals;
 use Biber::Constants;
 use Data::Dump qw( pp );
 use Digest::MD5 qw( md5_hex );
-use Encode;
 use Log::Log4perl qw( :no_extra_logdie_message );
 use List::Util qw( first );
+use Storable qw( dclone );
 
 my $logger = Log::Log4perl::get_logger('main');
 
@@ -26,18 +26,22 @@ Biber::Entry
 
     There are three types of field possible in an entry:
 
-    * data    - These are fields which derive directly from or are themselves fields in the
-                data source. Things like YEAR, MONTH, DAY etc. are such fields which are
-                derived from, for example, the DATE field. They are part of the original
-                data implicitly, derived from a field.
-    * derived - These are fields, often meta-information like labelname, labelalpha etc.
-                which are more removed from the data fields.
+    * raw  - These are direct copies of input fields with no processing performed on them.
+             Such fields are used for tool mode where we don't want to alter the fields as they
+             need to go back into the output as they are
+    * data - These are fields which derive directly from or are themselves fields in the
+             data souce. Things like YEAR, MONTH, DAY etc. are such fields which are derived from,
+             for example, the DATE field (which is itself a "raw" field). They are part of the
+             original data implicitly, derived from a "raw" field.
+    * other - These are fields, often meta-information like labelname, labelalpha etc. which are
+              more removed from the data fields.
 
     The reason for this division is largely the entry cloning required for the related entry and
     inheritance features. When we clone an entry or copy some fields from one entry to another
-    we generally don't want the "derived" category as such derived meta-fields will often need
-    to be re-created or ignored so we need to know which are the actual "data" fields to
-    copy/clone.
+    we generally don't want the "other" category as such derived meta-fields will often need
+    to be re-created or ignored so we need to know which are the actual "data" fields to copy/clone.
+    "raw" fields are important when we are writing bibtex format output (in tool mode for example)
+    since in such cases, we don't want to derive implicit fields like YEAR/MONTH from DATE.
 
 =cut
 
@@ -65,27 +69,20 @@ sub relclone {
   my $citekey = $self->get_field('citekey');
   my $secnum = $Biber::MASTER->get_current_section;
   my $section = $Biber::MASTER->sections->get_section($secnum);
-  my $dmh = Biber::Config->get_dm_helpers;
   if (my $relkeys = $self->get_field('related')) {
-    if ($logger->is_debug()) {# performance tune
-      $logger->debug("Found RELATED field in '$citekey' with contents " . join(',', @$relkeys));
-    }
+    $logger->debug("Found RELATED field in '$citekey' with contents " . join(',', @$relkeys));
     my @clonekeys;
     foreach my $relkey (@$relkeys) {
       # Resolve any alias
       my $nrelkey = $section->get_citekey_alias($relkey) // $relkey;
-      if ($logger->is_debug()) {# performance tune
-        $logger->debug("Resolved RELATED key alias '$relkey' to '$nrelkey'") if $relkey ne $nrelkey;
-        $logger->debug("Looking at RELATED key '$relkey'");
-      }
+      $logger->debug("Resolved RELATED key alias '$relkey' to '$nrelkey'") if $relkey ne $nrelkey;
       $relkey = $nrelkey;
+      $logger->debug("Looking at RELATED key '$relkey'");
 
       # Loop avoidance, in case we are back in an entry again in the guise of a clone
       # We can record the related clone but don't create it again
       if (my $ck = $section->get_keytorelclone($relkey)) {
-        if ($logger->is_debug()) {# performance tune
-          $logger->debug("Found RELATED key '$relkey' already has clone '$ck'");
-        }
+        $logger->debug("Found RELATED key '$relkey' already has clone '$ck'");
         push @clonekeys, $ck;
 
         # Save graph information if requested
@@ -95,35 +92,19 @@ sub relclone {
       }
       else {
         my $relentry = $section->bibentry($relkey);
-        # Digest::MD5 can't deal with straight UTF8 so encode it first (via NFC as this is "output")
-        my $clonekey = md5_hex(encode_utf8($relkey));
+        my $clonekey = md5_hex($relkey);
         push @clonekeys, $clonekey;
         my $relclone = $relentry->clone($clonekey);
-        if ($logger->is_debug()) {# performance tune
-          $logger->debug("Created new related clone for '$relkey' with clone key '$clonekey'");
-        }
+        $logger->debug("Created new related clone for '$relkey' with clone key '$clonekey'");
 
         # Set related clone options
         if (my $relopts = $self->get_field('relatedoptions')) {
-          # Check if this clone was also directly cited. If so, set skipbib/skipbiblist
-          # if they are unset as otherwise this entry would appear twice in bibliographies
-          # but with different keys.
-          if ($section->has_citekey($relkey)) {
-            $relopts = merge_entry_options($relopts, ['skipbib', 'skipbiblist']);
-          }
-
-          process_entry_options($clonekey, $relopts, $secnum);
+          process_entry_options($clonekey, $relopts);
           $relclone->set_datafield('options', $relopts);
         }
         else {
-          # related clone needs its own options plus all the dataonly opts, any conflicts and
-          # explicit options win
-
-          my $relopts = merge_entry_options(['skipbib', 'skiplab','skipbiblist','uniquename=false','uniquelist=false'], $relentry->get_field('options'));
-
-          # Preserve options already in the clone but add 'dataonly' options
-          process_entry_options($clonekey, $relopts, $secnum);
-          $relclone->set_datafield('options', $relopts);
+          process_entry_options($clonekey, [ 'skiplab', 'skipbiblist', 'uniquename=0', 'uniquelist=0' ]);
+          $relclone->set_datafield('options', [ 'dataonly' ]);
         }
 
         $section->bibentries->add_entry($clonekey, $relclone);
@@ -135,9 +116,7 @@ sub relclone {
         }
 
         # recurse so we can do cascading related entries
-        if ($logger->is_debug()) {# performance tune
-          $logger->debug("Recursing into RELATED entry '$clonekey'");
-        }
+        $logger->debug("Recursing into RELATED entry '$clonekey'");
         $relclone->relclone;
       }
     }
@@ -157,34 +136,21 @@ sub relclone {
 =cut
 
 sub clone {
-  my ($self, $newkey) = @_;
+  my $self = shift;
+  my $newkey = shift;
   my $new = new Biber::Entry;
-  my $dmh = Biber::Config->get_dm_helpers;
-
   while (my ($k, $v) = each(%{$self->{datafields}})) {
     $new->{datafields}{$k} = $v;
   }
+  while (my ($k, $v) = each(%{$self->{rawfields}})) {
+    $new->{rawfields}{$k} = $v;
+  }
   while (my ($k, $v) = each(%{$self->{origfields}})) {
     $new->{origfields}{$k} = $v;
   }
-
-  # clone derived date fields
-  foreach my $df ($dmh->{datefields}->@*) {
-    $df =~ s/date$//;
-    foreach my $dsf ('dateunspecified', 'datesplit', 'datejulian',
-                     'enddatejulian', 'dateapproximate', 'enddateapproximate',
-                     'dateuncertain', 'enddateuncertain', 'season', 'endseason',
-                     'era', 'endera') {
-      if (my $ds = $self->{derivedfields}{"$df$dsf"}) {
-        $new->{derivedfields}{"$df$dsf"} = $ds;
-      }
-    }
-  }
-
   # Need to add entrytype and datatype
   $new->{derivedfields}{entrytype} = $self->{derivedfields}{entrytype};
   $new->{derivedfields}{datatype} = $self->{derivedfields}{datatype};
-
   # put in key if specified
   if ($newkey) {
     $new->{derivedfields}{citekey} = $newkey;
@@ -210,13 +176,14 @@ sub notnull {
 =head2 set_labelname_info
 
   Record the labelname information. This is special
-  meta-information so we have a separate method for this
+  meta-information so we have a seperate method for this
   Takes a hash ref with the information.
 
 =cut
 
 sub set_labelname_info {
-  my ($self, $data) = @_;
+  my $self = shift;
+  my $data = shift;
   $self->{labelnameinfo} = $data;
   return;
 }
@@ -224,7 +191,7 @@ sub set_labelname_info {
 =head2 get_labelname_info
 
   Retrieve the labelname information. This is special
-  meta-information so we have a separate method for this
+  meta-information so we have a seperate method for this
   Returns a hash ref with the information.
 
 =cut
@@ -237,13 +204,14 @@ sub get_labelname_info {
 =head2 set_labelnamefh_info
 
   Record the fullhash labelname information. This is special
-  meta-information so we have a separate method for this
+  meta-information so we have a seperate method for this
   Takes a hash ref with the information.
 
 =cut
 
 sub set_labelnamefh_info {
-  my ($self, $data) = @_;
+  my $self = shift;
+  my $data = shift;
   $self->{labelnamefhinfo} = $data;
   return;
 }
@@ -251,7 +219,7 @@ sub set_labelnamefh_info {
 =head2 get_labelnamefh_info
 
   Retrieve the fullhash labelname information. This is special
-  meta-information so we have a separate method for this
+  meta-information so we have a seperate method for this
   Returns a hash ref with the information.
 
 =cut
@@ -264,13 +232,14 @@ sub get_labelnamefh_info {
 =head2 set_labeltitle_info
 
   Record the labeltitle information. This is special
-  meta-information so we have a separate method for this
+  meta-information so we have a seperate method for this
   Takes a hash ref with the information.
 
 =cut
 
 sub set_labeltitle_info {
-  my ($self, $data) = @_;
+  my $self = shift;
+  my $data = shift;
   $self->{labeltitleinfo} = $data;
   return;
 }
@@ -278,7 +247,7 @@ sub set_labeltitle_info {
 =head2 get_labeltitle_info
 
   Retrieve the labeltitle information. This is special
-  meta-information so we have a separate method for this
+  meta-information so we have a seperate method for this
   Returns a hash ref with the information.
 
 =cut
@@ -292,13 +261,14 @@ sub get_labeltitle_info {
 =head2 set_labeldate_info
 
   Record the labeldate information. This is special
-  meta-information so we have a separate method for this
+  meta-information so we have a seperate method for this
   Takes a hash ref with the information.
 
 =cut
 
 sub set_labeldate_info {
-  my ($self, $data) = @_;
+  my $self = shift;
+  my $data = shift;
   $self->{labeldateinfo} = $data;
   return;
 }
@@ -306,7 +276,7 @@ sub set_labeldate_info {
 =head2 get_labeldate_info
 
   Retrieve the labeldate information. This is special
-  meta-information so we have a separate method for this
+  meta-information so we have a seperate method for this
   Returns a hash ref with the information.
 
 =cut
@@ -325,7 +295,8 @@ sub get_labeldate_info {
 =cut
 
 sub set_field {
-  my ($self, $key, $val) = @_;
+  my $self = shift;
+  my ($key, $val) = @_;
   # All derived fields can be null
   $self->{derivedfields}{$key} = $val;
   return;
@@ -340,12 +311,15 @@ sub set_field {
 =cut
 
 sub get_field {
-  my ($self, $key) = @_;
+  my $self = shift;
+  my $key = shift;
   return undef unless $key;
   return $self->{datafields}{$key} //
-         $self->{derivedfields}{$key};
+         $self->{derivedfields}{$key} //
+         $self->{rawfields}{$key};
 }
 
+
 =head2 set_datafield
 
     Set a field which is in the .bib data file
@@ -353,11 +327,40 @@ sub get_field {
 =cut
 
 sub set_datafield {
-  my ($self, $key, $val) = @_;
+  my $self = shift;
+  my ($key, $val) = @_;
   $self->{datafields}{$key} = $val;
   return;
 }
 
+
+
+=head2 set_rawfield
+
+    Save a copy of the raw field from the datasource
+
+=cut
+
+sub set_rawfield {
+  my $self = shift;
+  my ($key, $val) = @_;
+  $self->{rawfields}{$key} = $val;
+  return;
+}
+
+=head2 get_rawfield
+
+    Get a raw field
+
+=cut
+
+sub get_rawfield {
+  my $self = shift;
+  my $key = shift;
+  return $self->{rawfields}{$key};
+}
+
+
 =head2 get_datafield
 
     Get a field that was in the original data file
@@ -365,7 +368,8 @@ sub set_datafield {
 =cut
 
 sub get_datafield {
-  my ($self, $key) = @_;
+  my $self = shift;
+  my $key = shift;
   return $self->{datafields}{$key};
 }
 
@@ -377,9 +381,11 @@ sub get_datafield {
 =cut
 
 sub del_field {
-  my ($self, $key) = @_;
+  my $self = shift;
+  my $key = shift;
   delete $self->{datafields}{$key};
   delete $self->{derivedfields}{$key};
+  delete $self->{rawfields}{$key};
   return;
 }
 
@@ -390,7 +396,8 @@ sub del_field {
 =cut
 
 sub del_datafield {
-  my ($self, $key) = @_;
+  my $self = shift;
+  my $key = shift;
   delete $self->{datafields}{$key};
   return;
 }
@@ -403,26 +410,11 @@ sub del_datafield {
 =cut
 
 sub field_exists {
-  my ($self, $key) = @_;
+  my $self = shift;
+  my $key = shift;
   return (exists($self->{datafields}{$key}) ||
-          exists($self->{derivedfields}{$key})) ? 1 : 0;
-}
-
-=head2 date_fields_exist
-
-    Check whether any parts of a date field exist when passed a datepart field name
-
-=cut
-
-sub date_fields_exist {
-  my ($self, $field) = @_;
-  my $t = $field =~ s/(?:end)?(?:year|month|day|hour|minute|second|season|timezone)$//r;
-  foreach my $dp ('year', 'month', 'day', 'hour', 'minute', 'second', 'season', 'timezone') {
-    if (exists($self->{datafields}{"$t$dp"}) or exists($self->{datafields}{"${t}end$dp"})) {
-      return 1;
-    }
-  }
-  return 0;
+          exists($self->{derivedfields}{$key}) ||
+          exists($self->{rawfields}{$key})) ? 1 : 0;
 }
 
 =head2 datafields
@@ -437,6 +429,18 @@ sub datafields {
   return sort keys %{$self->{datafields}};
 }
 
+=head2 rawfields
+
+    Returns a sorted array of the raw fields and contents
+
+=cut
+
+sub rawfields {
+  my $self = shift;
+  use locale;
+  return sort keys %{$self->{rawfields}};
+}
+
 =head2 count_datafields
 
     Returns the number of datafields
@@ -448,17 +452,6 @@ sub count_datafields {
   return keys %{$self->{datafields}};
 }
 
-=head2 derivedfields
-
-    Returns a sorted array of the fields which were added during processing
-
-=cut
-
-sub derivedfields {
-  my $self = shift;
-  use locale;
-  return sort keys %{$self->{derivedfields}};
-}
 
 =head2 fields
 
@@ -517,52 +510,38 @@ sub has_keyword {
 =cut
 
 sub add_warning {
-  my ($self, $warning) = @_;
-  push $self->{derivedfields}{warnings}->@*, $warning;
+  my $self = shift;
+  my $warning = shift;
+  push @{$self->{derivedfields}{warnings}}, $warning;
   return;
 }
 
 
 =head2 set_inherit_from
 
-    Inherit fields from first child entry
+    Inherit fields from parent entry
 
-    $entry->set_inherit_from($firstchild);
+    $entry->set_inherit_from($parententry);
 
     Takes a second Biber::Entry object as argument
-
-    The purpose here is to inherit fields so that sorting/labelling defaults
-    can be generated for set parents from the first child set member data, unless
-    the set parent itself already has some fields set that will do this. Set
-    parents only have certain fields output in the .bbl and those that output but
-    are not used in sorting/labelling data generation should not be inherited.
+    Tailored for set inheritance which is a straight 1:1 inheritance,
+    excluding certain fields for backwards compatibility
 
 =cut
 
 sub set_inherit_from {
-  my ($self, $parent) = @_;
-  my $dmh = Biber::Config->get_dm_helpers;
+  my $self = shift;
+  my $parent = shift;
 
   # Data source fields
   foreach my $field ($parent->datafields) {
     next if $self->field_exists($field); # Don't overwrite existing fields
-
-    # Annotations are allowed for set parents themselves so never inherit these.
-    # This can't be suppressed at .bbl writing as it is impossible to know there
-    # whether the field came from the parent or first child because inheritance
-    # is a low-level operation on datafields
-    next if fc($field) eq fc('annotation');
-
     $self->set_datafield($field, $parent->get_field($field));
   }
-
   # Datesplit is a special non datafield and needs to be inherited for any
   # validation checks which may occur later
-  foreach my $df ($dmh->{datefields}->@*) {
-    $df =~ s/date$//;
-    if (my $ds = $parent->get_field("${df}datesplit")) {
-      $self->set_field("${df}datesplit", $ds);
-    }
+  if (my $ds = $parent->get_field('datesplit')) {
+    $self->set_field('datesplit', $ds);
   }
   return;
 }
@@ -601,15 +580,24 @@ sub resolve_xdata {
         if (my $recurse_xdata = $xdatum_entry->get_field('xdata')) { # recurse
           $xdatum_entry->resolve_xdata($recurse_xdata);
         }
-        foreach my $field ($xdatum_entry->datafields()) { # set fields
-          next if $field eq 'ids'; # Never inherit aliases
-          $self->set_datafield($field, $xdatum_entry->get_field($field));
-
-          # Record graphing information if required
-          if (Biber::Config->getoption('output_format') eq 'dot') {
-            Biber::Config->set_graph('xdata', $xdatum_entry->get_field('citekey'), $entry_key, $field, $field);
+        # For tool mode with bibtex output we need to copy the raw fields
+        if (Biber::Config->getoption('tool') and
+            Biber::Config->getoption('output_format') eq 'bibtex') {
+          foreach my $field ($xdatum_entry->rawfields()) { # set raw fields
+            next if $field eq 'ids'; # Never inherit aliases
+            $self->set_rawfield($field, $xdatum_entry->get_rawfield($field));
+            $logger->debug("Setting field '$field' in entry '$entry_key' via XDATA");
           }
-          if ($logger->is_debug()) { # performance tune
+        }
+        else {
+          foreach my $field ($xdatum_entry->datafields()) { # set fields
+            next if $field eq 'ids'; # Never inherit aliases
+            $self->set_datafield($field, $xdatum_entry->get_field($field));
+
+            # Record graphing information if required
+            if (Biber::Config->getoption('output_format') eq 'dot') {
+              Biber::Config->set_graph('xdata', $xdatum_entry->get_field('citekey'), $entry_key, $field, $field);
+            }
             $logger->debug("Setting field '$field' in entry '$entry_key' via XDATA");
           }
         }
@@ -634,7 +622,6 @@ sub resolve_xdata {
 
 sub inherit_from {
   my ($self, $parent) = @_;
-  my $dmh = Biber::Config->get_dm_helpers;
 
   my $secnum = $Biber::MASTER->get_current_section;
   my $section = $Biber::MASTER->sections->get_section($secnum);
@@ -658,39 +645,29 @@ sub inherit_from {
 
   my $type        = $self->get_field('entrytype');
   my $parenttype  = $parent->get_field('entrytype');
-  my $inheritance = Biber::Config->getblxoption(undef, 'inheritance');
+  my $inheritance = Biber::Config->getblxoption('inheritance');
   my %processed;
   # get defaults
   my $defaults = $inheritance->{defaults};
   # global defaults ...
   my $inherit_all = $defaults->{inherit_all};
   my $override_target = $defaults->{override_target};
-  my $dignore = $defaults->{ignore};
-
   # override with type_pair specific defaults if they exist ...
-  foreach my $type_pair ($defaults->{type_pair}->@*) {
+  foreach my $type_pair (@{$defaults->{type_pair}}) {
     if (($type_pair->{source} eq '*' or $type_pair->{source} eq $parenttype) and
         ($type_pair->{target} eq '*' or $type_pair->{target} eq $type)) {
       $inherit_all = $type_pair->{inherit_all} if $type_pair->{inherit_all};
       $override_target = $type_pair->{override_target} if $type_pair->{override_target};
-      $dignore = $type_pair->{ignore} if defined($type_pair->{ignore});
     }
   }
 
   # First process any fields that have special treatment
-  foreach my $inherit ($inheritance->{inherit}->@*) {
+  foreach my $inherit (@{$inheritance->{inherit}}) {
     # Match for this combination of entry and crossref parent?
-    foreach my $type_pair ($inherit->{type_pair}->@*) {
+    foreach my $type_pair (@{$inherit->{type_pair}}) {
       if (($type_pair->{source} eq '*' or $type_pair->{source} eq $parenttype) and
           ($type_pair->{target} eq '*' or $type_pair->{target} eq $type)) {
-        foreach my $field ($inherit->{field}->@*) {
-          # Skip for fields in the per-entry noinerit datafield set
-          if (my $niset = Biber::Config->getblxoption($secnum, 'noinherit', undef, $target_key) and
-             exists($field->{target})) {
-            if (first {$field->{target} eq $_} $DATAFIELD_SETS{$niset}->@*) {
-              next;
-            }
-          }
+        foreach my $field (@{$inherit->{field}}) {
           next unless $parent->field_exists($field->{source});
           $processed{$field->{source}} = 1;
           # localise defaults according to field, if specified
@@ -702,20 +679,19 @@ sub inherit_from {
           # Set the field if it doesn't exist or override is requested
           elsif (not $self->field_exists($field->{target}) or
                  $field_override_target eq 'true') {
-            if ($logger->is_debug()) {# performance tune
-              $logger->debug("Entry '$target_key' is inheriting field '" .
-                             $field->{source}.
-                             "' as '" .
-                             $field->{target} .
-                             "' from entry '$source_key'");
+            $logger->debug("Entry '$target_key' is inheriting field '" .
+                           $field->{source}.
+                           "' as '" .
+                           $field->{target} .
+                           "' from entry '$source_key'");
+            # For tool mode with bibtex output we need to copy the raw fields
+            if (Biber::Config->getoption('tool') and
+                Biber::Config->getoption('output_format') eq 'bibtex') {
+              $self->set_rawfield($field->{target}, $parent->get_rawfield($field->{source}));
+            }
+            else {
+              $self->set_datafield($field->{target}, $parent->get_field($field->{source}));
             }
-
-            $self->set_datafield($field->{target}, $parent->get_field($field->{source}));
-
-            # Ignore uniqueness information tracking for this inheritance?
-            my $ignore = $inherit->{ignore} || $dignore;
-            Biber::Config->add_uniq_ignore($target_key, $field->{target}, $ignore);
-
             # Record graphing information if required
             if (Biber::Config->getoption('output_format') eq 'dot') {
               Biber::Config->set_graph('crossref', $source_key, $target_key, $field->{source}, $field->{target});
@@ -728,55 +704,38 @@ sub inherit_from {
 
   # Now process the rest of the (original data only) fields, if necessary
   if ($inherit_all eq 'true') {
-    my @fields = $parent->datafields;
-
-    # Special case - if the child has any Xdate datepart, don't inherit any Xdateparts
-    # from parent otherwise you can end up with rather broken dates in the child.
-    # Remove such fields before we start since it can't be done in the loop because
-    # as soon as one Xdatepart field has been inherited, no more will be.
-    my @filtered_fields;
-    foreach my $field (@fields) {
-      if (first {$_ eq $field} $dmh->{dateparts}->@*) {
-        next if $self->date_fields_exist($field);
-      }
-      push @filtered_fields, $field;
+    my @fields;
+    if (Biber::Config->getoption('tool')) {
+      @fields = $parent->rawfields;
+    }
+    else {
+      @fields = $parent->datafields;
     }
-    @fields = @filtered_fields;
-
     foreach my $field (@fields) {
-      # Skip for fields in the per-entry noinherit datafield set
-      if (my $niset = Biber::Config->getblxoption($secnum, 'noinherit', undef, $target_key)) {
-        if (first {$field eq $_} $DATAFIELD_SETS{$niset}->@*) {
-          next;
-        }
-      }
       next if $processed{$field}; # Skip if we have already dealt with this field above
-
       # Set the field if it doesn't exist or override is requested
       if (not $self->field_exists($field) or $override_target eq 'true') {
-        if ($logger->is_debug()) { # performance tune
-          $logger->debug("Entry '$target_key' is inheriting field '$field' from entry '$source_key'");
-        }
+            $logger->debug("Entry '$target_key' is inheriting field '$field' from entry '$source_key'");
+            # For tool mode with bibtex output we need to copy the raw fields
+            if (Biber::Config->getoption('tool') and
+                Biber::Config->getoption('output_format') eq 'bibtex') {
+              $self->set_rawfield($field, $parent->get_rawfield($field));
+            }
+            else {
+              $self->set_datafield($field, $parent->get_field($field));
+            }
 
-        $self->set_datafield($field, $parent->get_field($field));
-
-        # Ignore uniqueness information tracking for this inheritance?
-        Biber::Config->add_uniq_ignore($target_key, $field, $dignore);
-
-        # Record graphing information if required
-        if (Biber::Config->getoption('output_format') eq 'dot') {
-          Biber::Config->set_graph('crossref', $source_key, $target_key, $field, $field);
-        }
+            # Record graphing information if required
+            if (Biber::Config->getoption('output_format') eq 'dot') {
+              Biber::Config->set_graph('crossref', $source_key, $target_key, $field, $field);
+            }
       }
     }
   }
   # Datesplit is a special non datafield and needs to be inherited for any
   # validation checks which may occur later
-  foreach my $df ($dmh->{datefields}->@*) {
-    $df =~ s/date$//;
-    if (my $ds = $parent->get_field("${df}datesplit")) {
-      $self->set_field("${df}datesplit", $ds);
-    }
+  if (my $ds = $parent->get_field('datesplit')) {
+    $self->set_field('datesplit', $ds);
   }
 
   return;
@@ -809,7 +768,7 @@ L<https://github.com/plk/biber/issues>.
 
 =head1 COPYRIGHT & LICENSE
 
-Copyright 2009-2019 François Charette and Philip Kime, all rights reserved.
+Copyright 2009-2015 François Charette and Philip Kime, all rights reserved.
 
 This module is free software.  You can redistribute it and/or
 modify it under the terms of the Artistic License 2.0.
