@@ -1,8 +1,8 @@
 package Biber::Output::dot;
-use v5.24;
+use v5.16;
 use strict;
 use warnings;
-use parent qw(Biber::Output::base);
+use base 'Biber::Output::base';
 
 use Biber::Config;
 use Biber::Constants;
@@ -41,18 +41,18 @@ sub new {
   my $obj = shift;
   my $self = $class->SUPER::new($obj);
 
-  $self->{output_data}{HEAD} = <<~EOF;
-    digraph Biberdata {
-      compound = true;
-      edge [ arrowhead=open ];
-      graph [ style=filled, rankdir=LR ];
-      node [
-        fontsize=10,
-        fillcolor=white,
-        style=filled,
-        shape=box ];
+  $self->{output_data}{HEAD} = <<EOF;
+digraph Biberdata {
+  compound = true;
+  edge [ arrowhead=open ];
+  graph [ style=filled, rankdir=LR ];
+  node [
+    fontsize=10,
+    fillcolor=white,
+    style=filled,
+    shape=box ];
 
-    EOF
+EOF
 
   return $self;
 }
@@ -69,7 +69,10 @@ sub set_output_target_file {
   my $self = shift;
   my $dotfile = shift;
   $self->{output_target_file} = $dotfile;
-  return IO::File->new($dotfile, '>:encoding(UTF-8)');
+  my $enc_out;
+  $enc_out = ':encoding(UTF-8)';
+  my $DOTFILE = IO::File->new($dotfile, ">$enc_out");
+  $self->set_output_target($DOTFILE);
 }
 
 =head2 set_output_entry
@@ -113,9 +116,7 @@ sub output {
     $target = new IO::File '>-';
   }
 
-  if ($logger->is_debug()) {# performance tune
-    $logger->debug('Preparing final output using class ' . __PACKAGE__ . '...');
-  }
+  $logger->debug('Preparing final output using class ' . __PACKAGE__ . '...');
 
   $logger->info("Writing '$target_string' with encoding 'UTF-8'");
 
@@ -124,8 +125,8 @@ sub output {
   $in = 2; # indentation
   $i = ' '; # starting indentation
 
-  # Loop over sections, sort so we can run tests
-  foreach my $section (sort {$a->number <=> $b->number} $biber->sections->get_sections->@*) {
+  # Loop over sections.
+  foreach my $section (@{$biber->sections->get_sections}) {
     my $secnum = $section->number;
     if ($gopts->{section}) {
       $graph .= $i x $in . "subgraph \"cluster_section${secnum}\" {\n";
@@ -139,7 +140,7 @@ sub output {
     }
 
     # First create nodes/groups for entries
-    foreach my $be (sort {$a->get_field('citekey') cmp $b->get_field('citekey')} $section->bibentries->entries) {
+    foreach my $be ($section->bibentries->entries) {
       my $citekey = $be->get_field('citekey');
       $state->{$secnum}{"${secnum}/${citekey}"} = 1;
       my $et = uc($be->get_field('entrytype'));
@@ -147,7 +148,7 @@ sub output {
       # colour depends on whether cited, uncited, dataonly or key alias
       my $c = $section->has_citekey($citekey) ? '#a0d0ff' : '#deefff';
       if (my $options = $be->get_field('options')) {
-        $c = '#fdffd9' if $options =~ m/skip(?:bib|biblist|lab)/o;
+        $c = '#fdffd9' if $options =~ m/dataonly/o;
       }
       $c = '#a1edec' if $section->get_citekey_alias($citekey);
 
@@ -171,7 +172,7 @@ sub output {
 
       # Citekey aliases
       my $aliases = '';
-      foreach my $alias (sort $section->get_citekey_aliases) {
+      foreach my $alias ($section->get_citekey_aliases) {
         my $realkey = $section->get_citekey_alias($alias);
         if ($realkey eq $citekey) {
           $aliases .= "\\n$alias (alias)";
@@ -186,7 +187,7 @@ sub output {
         $graph .= $i x $in . "tooltip=\"$citekey ($et)\";\n";
         $graph .= $i x $in . "fillcolor=\"$c\";\n";
         $graph .= "\n";
-        foreach my $field (sort $be->datafields) {
+        foreach my $field ($be->datafields) {
           $graph .= $i x $in . "\"section${secnum}/${citekey}/${field}\" [ label=\"" . uc($field) . "\" ]\n";
         }
         $in -= 2;
@@ -251,9 +252,8 @@ sub _graph_related {
   if (my $gr = Biber::Config->get_graph('related')) {
 
     # related links
-    foreach my $f_entry (sort keys $gr->{clonetotarget}->%*) {
-      my $m = $gr->{clonetotarget}{$f_entry};
-      foreach my $t_entry (sort keys $m->%*) {
+    while (my ($f_entry, $m) = each %{$gr->{clonetotarget}}) {
+      foreach my $t_entry (keys %$m) {
         next unless $state->{$secnum}{"${secnum}/${f_entry}"};
         next unless $state->{$secnum}{"${secnum}/${t_entry}"};
 
@@ -269,9 +269,8 @@ sub _graph_related {
     }
 
     # clone links
-    foreach my $f_entry (sort keys $gr->{reltoclone}->%*) {
-      my $m = $gr->{reltoclone}{$f_entry};
-      foreach my $t_entry (sort keys $m->%*) {
+    while (my ($f_entry, $m) = each %{$gr->{reltoclone}}) {
+      foreach my $t_entry (keys %$m) {
         next unless $state->{$secnum}{"${secnum}/${f_entry}"};
         next unless $state->{$secnum}{"${secnum}/${t_entry}"};
 
@@ -292,8 +291,7 @@ sub _graph_related {
 sub _graph_xref {
   my $secnum = shift;
   if (my $gr = Biber::Config->get_graph('xref')) {
-    foreach my $f_entry (sort keys $gr->%*) {
-      my $t_entry = $gr->{$f_entry};
+    while (my ($f_entry, $t_entry) = each %$gr) {
       next unless $state->{$secnum}{"${secnum}/${f_entry}"};
       next unless $state->{$secnum}{"${secnum}/${t_entry}"};
 
@@ -324,26 +322,21 @@ sub _graph_inheritance {
   if (my $gr = Biber::Config->get_graph($type)) {
     # Show fields
     if ($gopts->{field}) {
-      foreach my $f_entry (sort keys $gr->%*) {
-        my $v = $gr->{$f_entry};
-        foreach my $f_field (sort keys $v->%*) {
-          my $w = $v->{$f_field};
-          foreach my $t_entry (sort keys $w->%*) {
-            foreach my $t_field ($w->{$t_entry}->@*) {
-              next unless $state->{$secnum}{"${secnum}/${f_entry}"};
-              next unless $state->{$secnum}{"${secnum}/${t_entry}"};
-              $graph_edges .= $i x $in . "\"section${secnum}/${f_entry}/${f_field}\" -> \"section${secnum}/${t_entry}/${t_field}\" [ penwidth=\"2.0\", color=\"${edgecolor}\", tooltip=\"${t_entry}/" . uc($t_field) . " inherited via " . uc($type) . " from ${f_entry}/" . uc($f_field) . "\" ]\n";
-            }
+      while (my ($f_entry, $v) = each %$gr) {
+        while (my ($f_field, $w) = each %$v) {
+          while (my ($t_entry, $t_field) = each %$w) {
+            next unless $state->{$secnum}{"${secnum}/${f_entry}"};
+            next unless $state->{$secnum}{"${secnum}/${t_entry}"};
+            $graph_edges .= $i x $in . "\"section${secnum}/${f_entry}/${f_field}\" -> \"section${secnum}/${t_entry}/${t_field}\" [ penwidth=\"2.0\", color=\"${edgecolor}\", tooltip=\"${t_entry}/" . uc($t_field) . " inherited via " . uc($type) . " from ${f_entry}/" . uc($f_field) . "\" ]\n";
           }
         }
       }
     }
     # Just show the entries, no fields
     else {
-      foreach my $f_entry (sort keys $gr->%*) {
-        my $v = $gr->{$f_entry};
-        foreach my $w (sort values $v->%*) {
-          foreach my $t_entry (sort keys $w->%*) {
+      while (my ($f_entry, $v) = each %$gr) {
+        while (my (undef, $w) = each %$v) {
+          while (my ($t_entry, undef) = each %$w) {
             next unless $state->{$secnum}{"${secnum}/${f_entry}"};
             next unless $state->{$secnum}{"${secnum}/${t_entry}"};
             next if $state->{edges}{"section${secnum}/${f_entry}"}{"section${secnum}/${t_entry}"};
@@ -363,6 +356,7 @@ __END__
 
 =head1 AUTHORS
 
+François Charette, C<< <firmicus at ankabut.net> >>
 Philip Kime C<< <philip at kime.org.uk> >>
 
 =head1 BUGS
@@ -372,7 +366,7 @@ L<https://github.com/plk/biber/issues>.
 
 =head1 COPYRIGHT & LICENSE
 
-Copyright 2012-2019 Philip Kime, all rights reserved.
+Copyright 2009-2015 François Charette and Philip Kime, all rights reserved.
 
 This module is free software.  You can redistribute it and/or
 modify it under the terms of the Artistic License 2.0.
