@@ -1,5 +1,5 @@
 package Biber::Section;
-use v5.24;
+use v5.16;
 use strict;
 use warnings;
 
@@ -27,7 +27,6 @@ sub new {
   $self->{relclonetokey} = {};
   $self->{relkeys} = {};
   $self->{allkeys} = 0;
-  $self->{allkeysnocite} = 0;
   $self->{citekeys} = [];
   $self->{citekeys_h} = {}; # For faster hash-based lookup of individual keys
   $self->{labelcache_l} = {};
@@ -35,13 +34,12 @@ sub new {
   $self->{everykey_lc} = {};
   $self->{bcfkeycache} = {};
   $self->{labelcache_v} = {};
+  $self->{sortcache} = [];
   $self->{dkeys} = {};
   $self->{keytods} = {};
   $self->{orig_order_citekeys} = [];
   $self->{undef_citekeys} = [];
-  $self->{nocite_citekeys} = {};
   $self->{citekey_alias} = {};
-  $self->{static_keys} = {};
   return $self;
 }
 
@@ -53,6 +51,7 @@ sub new {
 
 sub reset_caches {
   my $self = shift;
+  $self->{sortcache} = [];
   $self->{labelcache_l} = {};
   $self->{labelcache_v} = {};
   $self->{bcfkeycache} = {};
@@ -181,29 +180,6 @@ sub has_relclonetokey {
   return defined($self->{relclonetokey}{$key}) ? 1 : 0;
 }
 
-=head2 add_nocite
-
-    Adds a key to the list of those that came via \nocite
-
-=cut
-
-sub add_nocite {
-  my ($self, $key) = @_;
-  $self->{nocite_citekeys}{$key} = 1;
-  return;
-}
-
-=head2 get_nocite
-
-    Returns a boolean to say if a key came via \nocite
-
-=cut
-
-sub get_nocite {
-  my ($self, $key) = @_;
-  return (defined($self->{nocite_citekeys}{$key}) ? 1 : 0);
-}
-
 =head2 add_everykey
 
     Adds a datasource key to the section list of all datasource keys
@@ -243,18 +219,6 @@ sub has_everykey {
   return $self->{everykey}{$key} ? 1 : 0;
 }
 
-=head2 set_allkeys_nocite
-
-    Sets flag to say citekey '*' occurred through \nocite
-    We allow setting it to false too because it's useful in tests
-
-=cut
-
-sub set_allkeys_nocite {
-  my ($self, $val) = @_;
-  $self->{allkeysnocite} = $val;
-  return;
-}
 
 =head2 set_allkeys
 
@@ -269,17 +233,6 @@ sub set_allkeys {
   return;
 }
 
-=head2 is_allkeys_nocite
-
-    Checks flag which says citekey '*' occurred in via \nocite
-
-=cut
-
-sub is_allkeys_nocite {
-  my $self = shift;
-  return $self->{allkeysnocite};
-}
-
 =head2 is_allkeys
 
     Checks flag which says citekey '*' occurred in citekeys
@@ -342,7 +295,7 @@ sub del_bibentries {
 sub set_citekeys {
   my $self = shift;
   my $keys = shift;
-  map { $self->{citekeys_h}{$_} = 1} $keys->@*;
+  map { $self->{citekeys_h}{$_} = 1} @$keys;
   $self->{citekeys} = $keys;
   return;
 }
@@ -360,6 +313,7 @@ sub set_orig_order_citekeys {
   return;
 }
 
+
 =head2 get_citekeys
 
     Gets the citekeys of a Biber::Section object
@@ -369,7 +323,7 @@ sub set_orig_order_citekeys {
 
 sub get_citekeys {
   my $self = shift;
-  return $self->{citekeys}->@*;
+  return @{$self->{citekeys}};
 }
 
 =head2 get_static_citekeys
@@ -385,17 +339,6 @@ sub get_static_citekeys {
   return reduce_array($self->{citekeys}, $self->dynamic_set_keys);
 }
 
-=head2 has_cited_citekey
-
-    Returns true when $key was one of the actually cited keys in the section
-
-=cut
-
-sub has_cited_citekey {
-  my $self = shift;
-  my $key = shift;
-  return $self->{citekeys_h}{$key} ? 1 : 0;
-}
 
 =head2 add_undef_citekey
 
@@ -408,7 +351,7 @@ sub has_cited_citekey {
 sub add_undef_citekey {
   my $self = shift;
   my $key = shift;
-  push $self->{undef_citekeys}->@*, $key;
+  push @{$self->{undef_citekeys}}, $key;
   return;
 }
 
@@ -421,7 +364,7 @@ sub add_undef_citekey {
 
 sub get_undef_citekeys {
   my $self = shift;
-  return $self->{undef_citekeys}->@*;
+  return @{$self->{undef_citekeys}};
 }
 
 =head2 get_orig_order_citekeys
@@ -435,7 +378,7 @@ sub get_undef_citekeys {
 
 sub get_orig_order_citekeys {
   my $self = shift;
-  return $self->{orig_order_citekeys}->@*;
+  return @{$self->{orig_order_citekeys}};
 }
 
 =head2 has_citekey
@@ -462,8 +405,8 @@ sub del_citekey {
   my $self = shift;
   my $key = shift;
   return unless $self->has_citekey($key);
-  $self->{citekeys}            = [ grep {$_ ne $key} $self->{citekeys}->@* ];
-  $self->{orig_order_citekeys} = [ grep {$_ ne $key} $self->{orig_order_citekeys}->@* ];
+  $self->{citekeys}            = [ grep {$_ ne $key} @{$self->{citekeys}} ];
+  $self->{orig_order_citekeys} = [ grep {$_ ne $key} @{$self->{orig_order_citekeys}} ];
   delete $self->{citekeys_h}{$key};
   return;
 }
@@ -494,12 +437,13 @@ sub add_citekeys {
   foreach my $key (@keys) {
     next if $self->has_citekey($key);
     $self->{citekeys_h}{$key} = 1;
-    push $self->{citekeys}->@*, $key;
-    push $self->{orig_order_citekeys}->@*, $key;
+    push @{$self->{citekeys}}, $key;
+    push @{$self->{orig_order_citekeys}}, $key;
   }
   return;
 }
 
+
 =head2 set_citekey_alias
 
     Set citekey alias information
@@ -538,6 +482,7 @@ sub del_citekey_alias {
   return;
 }
 
+
 =head2 get_citekey_aliases
 
     Get a list of all citekey aliases for the section
@@ -546,9 +491,10 @@ sub del_citekey_alias {
 
 sub get_citekey_aliases {
   my $self = shift;
-  return ( keys $self->{citekey_alias}->%* );
+  return ( keys %{$self->{citekey_alias}} );
 }
 
+
 =head2 set_labelcache_v
 
     Sets the variable label disambiguation cache for a field
@@ -635,7 +581,7 @@ sub get_dynamic_set {
   my $self = shift;
   my $dkey = shift;
   if (my $set_members = $self->{dkeys}{$dkey}) {
-    return $set_members->@*;
+    return @$set_members;
   }
   else {
     return ();
@@ -650,7 +596,7 @@ sub get_dynamic_set {
 
 sub dynamic_set_keys {
   my $self = shift;
-  return [keys $self->{dkeys}->%*];
+  return [keys %{$self->{dkeys}}];
 }
 
 =head2 has_dynamic_sets
@@ -674,7 +620,7 @@ sub has_dynamic_sets {
 sub add_datasource {
   my $self = shift;
   my $source = shift;
-  push $self->{datasources}->@*, $source;
+  push @{$self->{datasources}}, $source;
   return;
 }
 
@@ -709,6 +655,34 @@ sub get_datasources {
 }
 
 
+=head2 add_sort_cache
+
+    Adds a scheme/keys pair to the sort cache:
+    [$scheme, $keys, $sortinitdata, $extraalphadata, $extrayeardata ]
+
+=cut
+
+sub add_sort_cache {
+  my $self = shift;
+  my $cacheitem = shift;
+  push @{$self->{sortcache}}, $cacheitem;
+  return;
+}
+
+
+=head2 get_sort_cache
+
+    Retrieves the sort cache
+
+=cut
+
+sub get_sort_cache {
+  my $self = shift;
+  return $self->{sortcache};
+}
+
+
+
 =head2 number
 
     Gets the section number of a Biber::Section object
@@ -736,7 +710,7 @@ L<https://github.com/plk/biber/issues>.
 
 =head1 COPYRIGHT & LICENSE
 
-Copyright 2009-2018 François Charette and Philip Kime, all rights reserved.
+Copyright 2009-2015 François Charette and Philip Kime, all rights reserved.
 
 This module is free software.  You can redistribute it and/or
 modify it under the terms of the Artistic License 2.0.
